package main

import (
	"fmt"
	"net/http"
	"strconv"

	"github.com/gin-gonic/gin"
	"github.com/jaswdr/faker"
)

// This is taken from our DB model
type video struct {
	ID          int    `json:"id"`
	Title       string `json:"title"`
	Description string `json:"description"`
	Filepath    string `json:"filepath"`
}

// videos slice to seed record video data.
var videos []video

func remove(slice []video, s int) []video {
	return append(slice[:s], slice[s+1:]...)
}

func main() {
	faker := faker.New()

	for i := 0; i < 10; i++ {
		fmt.Println(i)
		videos = append(videos, video{ID: i, Title: faker.RandomStringWithLength(20), Description: faker.Lorem().Sentence(50), Filepath: faker.File().AbsoluteFilePathForUnix(5)})
	}

	router := gin.Default()
	router.SetTrustedProxies(nil)

	router.POST("/upload", ReceiveFile)
	router.GET("/videos", getvideos)
	router.GET("/videos/:id", getvideoByID)
	router.POST("/videos", postvideos)
	router.DELETE("/videos", delvideos)
	router.DELETE("/videos/:id", delvideoByID)
	router.PATCH("/videos/:id", updatevideo)

	router.Run("localhost:8080")
}

func ReceiveFile(c *gin.Context) {
	c.IndentedJSON(http.StatusOK, "Received successfully")
}

// getvideos responds with the list of all videos as JSON.
func getvideos(c *gin.Context) {
	c.IndentedJSON(http.StatusOK, videos)
}

// postvideos adds an video from JSON received in the request body.
func postvideos(c *gin.Context) {
	var newvideo video

	// Call BindJSON to bind the received JSON to
	// newvideo.
	if err := c.BindJSON(&newvideo); err != nil {
		c.Status(http.StatusBadRequest)
		return
	}

	for _, a := range videos {
		if a == newvideo {
			c.Status(http.StatusBadRequest)
			return
		}
	}

	// Add the new video to the slice.
	videos = append(videos, newvideo)
	c.IndentedJSON(http.StatusCreated, newvideo)
}

// getvideoByID locates the video whose ID value matches the id
// parameter sent by the client, then returns that video as a response.
func getvideoByID(c *gin.Context) {
	inputId := c.Param("id")

	// Loop through the list of videos, looking for
	// an video whose ID value matches the parameter.
	for _, a := range videos {
		if strconv.Itoa(a.ID) == inputId {
			c.IndentedJSON(http.StatusOK, a)
			return
		}
	}
	c.IndentedJSON(http.StatusNotFound, gin.H{"message": "video not found"})
}

func delvideos(c *gin.Context) {
	videos = []video{}
	c.IndentedJSON(http.StatusOK, videos)
}

func delvideoByID(c *gin.Context) {
	inputId := c.Param("id")

	for s, a := range videos {
		if strconv.Itoa(a.ID) == inputId {
			videos = remove(videos, s)
			c.IndentedJSON(http.StatusCreated, videos)
			return
		}
	}
	c.IndentedJSON(http.StatusNotFound, gin.H{"message": "video not found"})
}

func updatevideo(c *gin.Context) {
	inputId := c.Param("id")

	var newvideo video

	if err := c.BindJSON(&newvideo); err != nil {
		return
	}

	for s, a := range videos {
		if strconv.Itoa(a.ID) == inputId {
			videos = remove(videos, s)
			videos = append(videos, newvideo)
			c.IndentedJSON(http.StatusCreated, newvideo)
			return
		}
	}
	c.IndentedJSON(http.StatusNotFound, gin.H{"message": "video not found"})
}
